/*
 *  nautilus-filesharing.c - File sharing for Nautilus
 *  (based on nautilus-vcs from Dave Camp) 
 * 
 *  Copyright (c) 2004,2005 		Mandrakesoft
 *  		  2005,2006,2007,2008 	Mandriva
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Authors: Dave Camp <dave@ximian.com>
 *  	     Frederic Crozat <fcrozat@mandriva.com>
 * 
 */

#include <config.h>

#include <pthread.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <gtk/gtk.h>

#include <libnautilus-extension/nautilus-extension-types.h>
#include <libnautilus-extension/nautilus-menu-provider.h>
#include <libnautilus-extension/nautilus-property-page-provider.h>
#include <libnautilus-extension/nautilus-file-info.h>
#include <libnautilus-extension/nautilus-info-provider.h>

#define FILESHARE_LIST_PROGRAM   "/usr/sbin/filesharelist"
#define FILESHARE_SET_PROGRAM    "/usr/sbin/fileshareset"
#define FILESHARE_CONFIG_PROGRAM "/usr/sbin/diskdrake --fileshare"
#define FILESHARE_CONFIG_FILE    "/etc/security/fileshare.conf"

#define GNOME_PAD          8
#define GNOME_PAD_SMALL    4
#define GNOME_PAD_BIG      12


#include <glib/gi18n-lib.h>

static int can_share_directories = -1;
static GHashTable *shared_directories;
static GHashTable *queried_directories_files;
static GList *shared_tabs_list = NULL;
static pthread_t   fileshare_thread = (pthread_t) NULL;
static GFileMonitor* fileshare_monitor;

static int 
refresh_shared_directories(char * command) {
	static GHashTable* hash;
	char buffer[4096];
	FILE *file;
	GFile *gfile;
	int result;

	buffer[0] = '\0';

	hash = g_hash_table_new (g_str_hash, g_str_equal);
	file = popen (command, "r");
	if (file != NULL) {
		while (fgets (buffer, sizeof(buffer) - strlen(buffer), file) != NULL) {
			if (buffer[strlen(buffer) -1] == '\n') {
				buffer[strlen(buffer) -1] = '\0';
			}
			if ((strlen(buffer) == 0) || (buffer[0] != '/')) {
				continue;
			}
			gfile = g_file_new_for_path (buffer);
			g_hash_table_insert (hash, g_file_get_uri (gfile), (gpointer)1);
			g_object_unref (gfile);
		}
		result = pclose (file);
		if (result == 0) {
			if (shared_directories != NULL) {
				g_hash_table_destroy (shared_directories);
			}
			shared_directories = hash;
		}
		g_hash_table_foreach (queried_directories_files, (GHFunc) nautilus_file_info_invalidate_extension_info, NULL);
		return result;
	}
	return -1;
}

static void
update_share_tab (GtkWidget *notebook);

static void 
update_all_share_tabs () {
	g_list_foreach (shared_tabs_list, (GFunc) update_share_tab, NULL);
}

static void *
fileshare_config_exec (void *arg)
{
	g_spawn_command_line_sync (FILESHARE_CONFIG_PROGRAM, NULL, NULL, NULL, NULL);

	fileshare_thread = (pthread_t) NULL;
	g_idle_add ((GSourceFunc)update_all_share_tabs, NULL);
	
	pthread_exit (NULL);

	/* compilation on Solaris warns of no return 
	   value on non-void function...so....*/
	return (void *) 0;
}

static void disable_share_property (gpointer user_data) {
	gtk_widget_set_sensitive(GTK_WIDGET (user_data), FALSE);
}

static void
fileshare_config_has_changed_callback (GFileMonitor *monitor,
				       GFile *child,
				       GFile *other_file,
				       GFileMonitorEvent event_type,
				       gpointer user_data)
{
	gboolean old_value = can_share_directories;
	can_share_directories = (geteuid() == 0) || refresh_shared_directories(FILESHARE_LIST_PROGRAM " 2> /dev/null") == 0;
	if ((old_value > 0) && !can_share_directories && shared_directories != NULL) {
		g_hash_table_foreach (queried_directories_files, (GHFunc) nautilus_file_info_invalidate_extension_info, NULL);
	}

	if (fileshare_thread == (pthread_t) NULL) {
		update_all_share_tabs ();
	}
}

static gboolean
nautilus_filesharing_can_share_directories (void)
{
	if (can_share_directories == -1) {
		fileshare_config_has_changed_callback (NULL, NULL, NULL, 0, NULL);
	}
	return can_share_directories;
}


/* NautilusFilesharing provider. */

#define NAUTILUS_TYPE_FILESHARING  (nautilus_filesharing_get_type ())
#define NAUTILUS_FILESHARING(o)    (G_TYPE_CHECK_INSTANCE_CAST ((o), NAUTILUS_TYPE_FILESHARING, NautilusFilesharing))

typedef struct {
	GObject parent_slot;
} NautilusFilesharing;

typedef struct {
	GObjectClass parent_slot;
} NautilusFilesharingClass;

static GType nautilus_filesharing_get_type      (void);
static void  nautilus_filesharing_register_type (GTypeModule *module);

static GObjectClass *parent_class;

static void
nautilus_filesharing_instance_init (NautilusFilesharing *fs)
{
}

static void
nautilus_filesharing_class_init (NautilusFilesharingClass *class)
{
        parent_class = g_type_class_peek_parent (class);
}

static GType filesharing_type = 0;

static GType
nautilus_filesharing_get_type (void)
{
	return filesharing_type;
}


static void launch_filesharing_callback (GtkWidget               *widget,
					 GtkNotebook             *share_notebook) 
{
	g_list_foreach (shared_tabs_list, (GFunc) disable_share_property, NULL);

	pthread_create (&fileshare_thread, NULL, fileshare_config_exec, widget);
	
}

static gboolean
nautilus_filesharing_is_shared (NautilusFileInfo *file)
{
	char * local_uri;
	gboolean return_value;

	if (file != NULL && nautilus_file_info_is_directory (file) && nautilus_filesharing_can_share_directories() && shared_directories != NULL) {
		local_uri = nautilus_file_info_get_uri (file);
		return_value = (gboolean) g_hash_table_lookup (shared_directories, local_uri);
		g_free (local_uri);
		return return_value;
	}
	else {
		return FALSE;
	}
}

static int
nautilus_filesharing_set_shared (NautilusFileInfo *file, gboolean shared)
{
	char *local_uri, *local_path, *option, *command;
	GFile *gfile;
	int result;

	local_uri = nautilus_file_info_get_uri (file);
	gfile = g_file_new_for_uri (local_uri);
	g_free (local_uri);
	local_path = g_file_get_path (gfile);
	g_object_unref (gfile);
	if (shared) {
		option = " --add ";
	}
	else {
		option = " --remove ";
	}
	command = g_strconcat (FILESHARE_SET_PROGRAM, option, local_path, " 2>/dev/null", NULL);
	result = refresh_shared_directories(command);

	nautilus_file_info_invalidate_extension_info (file);
	g_free (command);

	g_free (local_path);
	
	return result;
}

static void
share_changed (GtkWidget *not_shared_radio_button, NautilusFileInfo * file) {
	int not_shared;

	g_assert (GTK_IS_TOGGLE_BUTTON (not_shared_radio_button));
	g_assert (NAUTILUS_IS_FILE_INFO (file));

	not_shared = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(not_shared_radio_button));
	if (g_object_get_data (G_OBJECT(not_shared_radio_button),"sharing_in_progress") == NULL) {
		int result;
		GtkWidget *dialog;
		result = nautilus_filesharing_set_shared (file, !not_shared);
		if (result != 0) {
			g_object_set_data (G_OBJECT (not_shared_radio_button), "sharing_in_progress", "true");
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (not_shared_radio_button), !not_shared);
			g_object_set_data (G_OBJECT (not_shared_radio_button), "sharing_in_progress", NULL);

			dialog = gtk_message_dialog_new (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (not_shared_radio_button))), GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, _("Error while running " FILESHARE_SET_PROGRAM));
			gtk_dialog_run (GTK_DIALOG (dialog));
			gtk_widget_destroy (dialog);

		}
	}	
}
 
static void
update_share_tab (GtkWidget *notebook) {
	GFile *directory_file, *home_file;
	char *directory_uri_string;
	gint page_number;
	NautilusFileInfo *file;
	GtkWidget *not_shared_radio_button, *shared_radio_button;

	file = g_object_get_data (G_OBJECT (notebook), "file");
	not_shared_radio_button = g_object_get_data (G_OBJECT (notebook), "not_shared_radio_button");
	shared_radio_button = g_object_get_data (G_OBJECT (notebook), "shared_radio_button");

	if ((file == NULL) || (nautilus_file_info_is_directory(file) != TRUE)) 
		return;

	directory_uri_string = nautilus_file_info_get_uri (file);
	directory_file = g_file_new_for_uri (directory_uri_string);
	g_free (directory_uri_string);
	home_file = g_file_new_for_path (g_get_home_dir ());

	if (g_file_equal (home_file, directory_file)
	    || g_file_has_prefix (directory_file, home_file)) {
		if (g_file_test(FILESHARE_LIST_PROGRAM,G_FILE_TEST_EXISTS)) {
			if (nautilus_filesharing_can_share_directories()) {
				page_number = 0;
			}
			else {
				page_number = 1;
			}
		}
		else {
			page_number = 2;
		}
	}
	else {
		page_number = 3;
	}

	gtk_widget_set_sensitive(notebook, 
				 (fileshare_thread == (pthread_t) NULL));
	gtk_notebook_set_current_page(GTK_NOTEBOOK (notebook), page_number);
	if (g_object_get_data (G_OBJECT(not_shared_radio_button),
			       "sharing_in_progress") == NULL) {
		g_object_set_data (G_OBJECT (not_shared_radio_button), 
				   "sharing_in_progress", "true");
		if (nautilus_filesharing_is_shared(file)) {
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (shared_radio_button), TRUE);
		} else {
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (not_shared_radio_button), TRUE);
		}
		g_object_set_data(G_OBJECT (not_shared_radio_button), 
				  "sharing_in_progress", NULL);
	}
	else {
		if (nautilus_filesharing_is_shared(file)) {
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (shared_radio_button), TRUE);
		} else {
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (not_shared_radio_button), TRUE);
		}
	}	

	g_object_unref (directory_file);
	g_object_unref (home_file);	
}

static gboolean
notebook_destroy (GtkWidget *widget, gpointer user_data) {
	g_object_unref (g_object_get_data (G_OBJECT (widget), "file"));
	g_object_unref (g_object_get_data (G_OBJECT (widget), "not_shared_radio_button"));
	g_object_unref (g_object_get_data (G_OBJECT (widget), "shared_radio_button"));
	shared_tabs_list = g_list_remove (shared_tabs_list, widget);
	return FALSE;
}


/* Share properties */
static GtkWidget *
create_property_page (NautilusFileInfo *file)
{
	GtkWidget *vbox, *filesharing_button, *hbox;
	GtkWidget *notebook;
	GtkWidget *not_shared_radio_button, *shared_radio_button;
	GtkWidget *prompt, *separator_line;

	g_return_val_if_fail (nautilus_file_info_is_directory(file) == TRUE, NULL);
	notebook = gtk_notebook_new ();
	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook), FALSE);
	gtk_notebook_set_show_border (GTK_NOTEBOOK (notebook), FALSE);

	gtk_widget_show (notebook);

	/* create general sharing page */
	vbox = gtk_vbox_new (FALSE, 0);
        gtk_widget_show_all (vbox);
        gtk_container_set_border_width (GTK_CONTAINER (vbox), GNOME_PAD);
        gtk_notebook_append_page (GTK_NOTEBOOK(notebook), vbox, gtk_label_new ("shared authorized"));

	not_shared_radio_button = gtk_radio_button_new_with_label (NULL, _("Not shared"));
	gtk_box_pack_start (GTK_BOX (vbox), 
			    GTK_WIDGET (not_shared_radio_button), 
			    FALSE, FALSE, 0);

	shared_radio_button = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON (not_shared_radio_button), _("Shared"));
	gtk_widget_show (shared_radio_button);

	gtk_box_pack_start (GTK_BOX (vbox), 
			    GTK_WIDGET (shared_radio_button), 
			    FALSE, FALSE, 0);
				
	g_signal_connect (not_shared_radio_button,
			  "toggled",
			  G_CALLBACK (share_changed),
			  file);
								  
	gtk_widget_show (GTK_WIDGET (not_shared_radio_button));

	prompt = gtk_label_new (_("Sharing this directory makes it available under\nLinux/Unix (NFS) and Windows (Samba)"));
	gtk_label_set_justify (GTK_LABEL (prompt), GTK_JUSTIFY_LEFT);
        gtk_label_set_line_wrap (GTK_LABEL (prompt), TRUE);
	gtk_misc_set_alignment (GTK_MISC (prompt), 0, 0);
        gtk_widget_show (prompt);
	gtk_box_pack_start (GTK_BOX (vbox), prompt, FALSE, FALSE, GNOME_PAD_BIG);

	separator_line = gtk_hseparator_new ();
        gtk_widget_show (separator_line);
        gtk_box_pack_start (GTK_BOX (vbox), separator_line, FALSE, FALSE, GNOME_PAD);

	prompt = gtk_label_new (_( "You can also reconfigure file sharing authorisation"));
	gtk_misc_set_alignment (GTK_MISC (prompt), 0, 0);
	gtk_label_set_justify (GTK_LABEL (prompt), GTK_JUSTIFY_LEFT);
        gtk_label_set_line_wrap (GTK_LABEL (prompt), TRUE);
        gtk_widget_show (prompt);
	gtk_box_pack_start (GTK_BOX (vbox), prompt, FALSE, FALSE, GNOME_PAD);

	filesharing_button = gtk_button_new_with_label (_("Configure File Sharing"));
	gtk_widget_show (filesharing_button);
	hbox = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_start (GTK_BOX (hbox), filesharing_button, FALSE, FALSE, 0);
	gtk_widget_show (hbox);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, GNOME_PAD); 



	g_signal_connect(filesharing_button, "clicked", 
			 G_CALLBACK (launch_filesharing_callback), notebook);
	gtk_notebook_reorder_child (GTK_NOTEBOOK (notebook), vbox, 0);

	/* create "not configured" sharing page */
	vbox = gtk_vbox_new (FALSE, 0);
        gtk_widget_show (vbox);
        gtk_container_set_border_width (GTK_CONTAINER (vbox), GNOME_PAD);
        gtk_notebook_append_page (GTK_NOTEBOOK(notebook), vbox, gtk_label_new ("shared not configured"));


	prompt = gtk_label_new (_( "You need to be authorized to share directories"));
	gtk_misc_set_alignment (GTK_MISC (prompt), 0, 0);
	gtk_label_set_justify (GTK_LABEL (prompt), GTK_JUSTIFY_LEFT);
        gtk_label_set_line_wrap (GTK_LABEL (prompt), TRUE);
        gtk_widget_show (prompt);
	gtk_box_pack_start (GTK_BOX (vbox), prompt, FALSE, FALSE, 0);

	filesharing_button = gtk_button_new_with_label (_("Configure File Sharing"));
	gtk_widget_show (filesharing_button);
	hbox = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_start (GTK_BOX (hbox), filesharing_button, FALSE, FALSE, 0);
	gtk_widget_show (hbox);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, GNOME_PAD_SMALL);  
	g_signal_connect(filesharing_button, "clicked", 
			 G_CALLBACK (launch_filesharing_callback), notebook);

	gtk_notebook_reorder_child (GTK_NOTEBOOK (notebook), vbox, 1);
				
	/* create "error" sharing page */
 	vbox = gtk_vbox_new (FALSE, 0);
        gtk_widget_show (vbox);
        gtk_container_set_border_width (GTK_CONTAINER (vbox), GNOME_PAD);
        gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, gtk_label_new ("shared error"));

	prompt = gtk_label_new (_( "Error running `filesharelist'. Check if installed and in\n$PATH or /usr/sbin."));
	gtk_misc_set_alignment (GTK_MISC (prompt), 0, 0);
	gtk_label_set_justify (GTK_LABEL (prompt), GTK_JUSTIFY_LEFT);
        gtk_label_set_line_wrap (GTK_LABEL (prompt), TRUE);
        gtk_widget_show (prompt);
	gtk_box_pack_start (GTK_BOX (vbox), prompt, FALSE, FALSE, 0);

	gtk_notebook_reorder_child (GTK_NOTEBOOK (notebook), vbox, 2);

	/* create "home only" sharing page */
	vbox = gtk_vbox_new (FALSE, 0);
        gtk_widget_show (vbox);
        gtk_container_set_border_width (GTK_CONTAINER (vbox), GNOME_PAD);
        gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, gtk_label_new ("home only"));

	prompt = gtk_label_new (_( "Only directories in your Home can be shared."));
	gtk_misc_set_alignment (GTK_MISC (prompt), 0, 0);
	gtk_label_set_justify (GTK_LABEL (prompt), GTK_JUSTIFY_LEFT);
        gtk_label_set_line_wrap (GTK_LABEL (prompt), TRUE);
        gtk_widget_show (prompt);
	gtk_box_pack_start (GTK_BOX (vbox), prompt, FALSE, FALSE, 0);

	gtk_notebook_reorder_child (GTK_NOTEBOOK (notebook), vbox, 3);

	g_object_ref (not_shared_radio_button);
	g_object_set_data (G_OBJECT (notebook),
			"not_shared_radio_button",
			not_shared_radio_button);

	g_object_ref (shared_radio_button);
	g_object_set_data (G_OBJECT (notebook),
			"shared_radio_button",
			shared_radio_button);

	g_object_ref (file);
	g_object_set_data (G_OBJECT (notebook),
			"file",
			file);

	update_share_tab(notebook);

	g_signal_connect (notebook,
			  "destroy",
			  G_CALLBACK (notebook_destroy),
			  NULL);
	
	return notebook;
}

static void
show_share(NautilusMenuItem *item,
	   gpointer user_data)
{
	GtkDialog *dialog = GTK_DIALOG (gtk_dialog_new_with_buttons (_("Share"),
			NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_CLOSE, GTK_RESPONSE_NONE, NULL));

	GtkWidget *page = create_property_page (user_data);

	shared_tabs_list = g_list_append (shared_tabs_list, page);
	

	gtk_container_add (GTK_CONTAINER (dialog->vbox), page);

	g_signal_connect_swapped (dialog,
                             "response", 
                             G_CALLBACK (gtk_widget_destroy),
                             dialog);

	gtk_widget_show (GTK_WIDGET (dialog));
}


/* nautilus_menu_provider_get_file_items
 *  
 * This function is called by Nautilus when it wants context menu
 * items from the extension.
 *
 * This function is called in the main thread before a context menu
 * is shown, so it should return quickly.
 * 
 * The function should return a GList of allocated NautilusMenuItem
 * items.
 */
static GList *
nautilus_filesharing_get_file_items (NautilusMenuProvider *provider,
			     GtkWidget *window,
			     GList *files)
{
	NautilusMenuItem *item;
	GList *l;
	GList *items = NULL;
	NautilusFileInfo *file;
	
	if (nautilus_filesharing_can_share_directories () == FALSE)
		return NULL;

	for (l = files ; l != NULL; l = l->next) {
		file = NAUTILUS_FILE_INFO (l->data);
		if (nautilus_file_info_is_directory (file)) {
			item = nautilus_menu_item_new ("NautilusFilesharing::run",
					       _("Share"),
					       _("Share"),
					       NULL);
			g_signal_connect (item, "activate",
				  G_CALLBACK (show_share),
				  file);
			items = g_list_append (items, item);
		}
	}

	return items;
}

static void 
nautilus_filesharing_menu_provider_iface_init (NautilusMenuProviderIface *iface)
{
	iface->get_file_items = nautilus_filesharing_get_file_items;
}

static GList *
nautilus_filesharing_get_property_pages (NautilusPropertyPageProvider *provider,
				 GList *files)
{
	NautilusFileInfo *file;
	GList *pages;
	NautilusPropertyPage *page;
	GtkWidget *tab;

	/* Only show the property page if 1 file is selected */
	if (!files || files->next != NULL) {
		return NULL;
	}

	pages = NULL;

	file = NAUTILUS_FILE_INFO (files->data);
	if (nautilus_file_info_is_directory (file)) {
		tab = create_property_page (file);
		page = nautilus_property_page_new
			("NautilusFilesharing::property_page", 
			 gtk_label_new (_("Share")), tab);
		shared_tabs_list = g_list_append (shared_tabs_list, tab);
		pages = g_list_append (pages, page);
	}


	return pages;
}

static void 
nautilus_filesharing_property_page_provider_iface_init (NautilusPropertyPageProviderIface *iface)
{
	iface->get_pages = nautilus_filesharing_get_property_pages;
}

static void
file_info_weak_notify (gpointer user_data,
                       NautilusFileInfo *file)
{
        g_hash_table_remove (queried_directories_files, file);
}


/* Implementation of the NautilusInfoProvider interface */

/* nautilus_info_provider_update_file_info 
 * This function is called by Nautilus when it wants the extension to 
 * fill in data about the file.  It passes a NautilusFileInfo object,
 * which the extension can use to read data from the file, and which
 * the extension should add data to.
 *
 * If the data can be added immediately (without doing blocking IO), 
 * the extension can do so, and return NAUTILUS_OPERATION_COMPLETE.  
 * In this case the 'update_complete' and 'handle' parameters can be 
 * ignored.
 * 
 * If waiting for the deata would block the UI, the extension should
 * perform the task asynchronously, and return 
 * NAUTILUS_OPERATION_IN_PROGRESS.  The function must also set the 
 * 'handle' pointer to a value unique to the object, and invoke the
 * 'update_complete' closure when the update is done.
 * 
 * If the extension encounters an error, it should return 
 * NAUTILUS_OPERATION_FAILED.
 */
static NautilusOperationResult
nautilus_filesharing_update_file_info (NautilusInfoProvider *provider,
			       NautilusFileInfo *file,
			       GClosure *update_complete,
			       NautilusOperationHandle **handle)
{
	if (!g_hash_table_lookup (queried_directories_files, file)) {
		g_hash_table_insert (queried_directories_files, file, (gpointer)1);
		 /* Arrange to unref when the object goes away */
                g_object_weak_ref (G_OBJECT (file),
                                   (GWeakNotify)file_info_weak_notify,
                                   NULL);

	}
	if (nautilus_filesharing_is_shared (file)) {
		nautilus_file_info_add_emblem (file, "shared");	
	}
	return NAUTILUS_OPERATION_COMPLETE;
}

static void 
nautilus_filesharing_info_provider_iface_init (NautilusInfoProviderIface *iface)
{
	iface->update_file_info = nautilus_filesharing_update_file_info;
}

static void
nautilus_filesharing_register_type (GTypeModule *module)
{
	static const GTypeInfo info = {
                sizeof (NautilusFilesharingClass),
                (GBaseInitFunc) NULL,
                (GBaseFinalizeFunc) NULL,
                (GClassInitFunc) nautilus_filesharing_class_init,
                NULL,
                NULL,
                sizeof (NautilusFilesharing),
                0,
                (GInstanceInitFunc) nautilus_filesharing_instance_init,
        };

	filesharing_type = g_type_module_register_type (module, 
						G_TYPE_OBJECT,
						"NautilusFilesharing",
						&info, 0);

	static const GInterfaceInfo menu_provider_iface_info = {
		(GInterfaceInitFunc) nautilus_filesharing_menu_provider_iface_init,
		NULL,
		NULL
	};

	static const GInterfaceInfo property_page_provider_iface_info = {
		(GInterfaceInitFunc) nautilus_filesharing_property_page_provider_iface_init,
		NULL,
		NULL
	};

	static const GInterfaceInfo info_provider_iface_info = {
		(GInterfaceInitFunc) nautilus_filesharing_info_provider_iface_init,
		NULL,
		NULL
	};

	g_type_module_add_interface (module,
				     filesharing_type,
				     NAUTILUS_TYPE_MENU_PROVIDER,
				     &menu_provider_iface_info);

	g_type_module_add_interface (module,
				     filesharing_type,
				     NAUTILUS_TYPE_PROPERTY_PAGE_PROVIDER,
				     &property_page_provider_iface_info);

	g_type_module_add_interface (module,
				     filesharing_type,
				     NAUTILUS_TYPE_INFO_PROVIDER,
				     &info_provider_iface_info);


}

static void 
nautilus_filesharing_initialize_share_monitor() 
{
	GFile *fileshare_file;
	
	fileshare_file = g_file_new_for_path (FILESHARE_CONFIG_FILE);
	fileshare_monitor = g_file_monitor_file (fileshare_file, 
			       G_FILE_MONITOR_NONE,
			       NULL, 
			       NULL);
	if (fileshare_monitor) {
		g_signal_connect (fileshare_monitor, "changed", (GCallback)fileshare_config_has_changed_callback, NULL);
	}

	g_object_unref (fileshare_file);
}


/* Extension module functions.  These functions are defined in 
 * nautilus-extensions-types.h, and must be implemented by all 
 * extensions. */

/* Initialization function.  In addition to any module-specific 
 * initialization, any types implemented by the module should 
 * be registered here. */
void
nautilus_module_initialize (GTypeModule  *module)
{

	queried_directories_files = g_hash_table_new (g_direct_hash,
				g_direct_equal);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	nautilus_filesharing_register_type (module);
	nautilus_filesharing_initialize_share_monitor ();
}

/* Perform module-specific shutdown. */
void
nautilus_module_shutdown   (void)
{
	if (fileshare_monitor != NULL) {
		g_signal_handlers_disconnect_by_func (fileshare_monitor, fileshare_config_has_changed_callback, NULL);
		g_file_monitor_cancel (fileshare_monitor);
		g_object_unref (fileshare_monitor);
	}
	g_hash_table_destroy (queried_directories_files);
	g_list_foreach (shared_tabs_list, (GFunc) g_object_unref, NULL);
	g_list_free (shared_tabs_list);
	g_hash_table_destroy (shared_directories);
}

/* List all the extension types.  */
void 
nautilus_module_list_types (const GType **types,
			    int          *num_types)
{
	static GType type_list[2];
	
	type_list[0] = NAUTILUS_TYPE_FILESHARING;

	*types = type_list;
	*num_types = 1;
}
